"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
exports.__esModule = true;
exports.introspectMachine = exports.getMatchesStates = void 0;
require("colors");
var utils_1 = require("xstate/lib/utils");
var traversalUtils_1 = require("./traversalUtils");
exports.getMatchesStates = function (machine) {
    var allStateNodes = machine.stateIds.map(function (id) {
        return machine.getStateNodeById(id);
    });
    var states = allStateNodes.reduce(function (arr, node) {
        return __spreadArrays(arr, utils_1.toStatePaths(utils_1.pathToStateValue(node.path)).map(function (path) {
            return path.join('.');
        }));
    }, []);
    return states;
};
var makeSubStateFromNode = function (node, rootNode, nodeMaps) {
    var nodeFromMap = nodeMaps[node.id];
    var stateNode = rootNode.getStateNodeById(node.id);
    var targets = traversalUtils_1.getTransitionsFromNode(stateNode);
    return {
        sources: Array.from(nodeFromMap.sources)
            .filter(Boolean)
            .map(function (event) { return "'" + event + "'"; })
            .join(' | ') || 'never',
        targets: Array.from(targets)
            .filter(Boolean)
            .map(function (event) { return "'" + event + "'"; })
            .join(' | ') || 'never',
        states: Array.from(nodeFromMap.children).reduce(function (obj, child) {
            var _a;
            var childNode = rootNode.getStateNodeById(child);
            return __assign(__assign({}, obj), (_a = {}, _a[childNode.key] = makeSubStateFromNode(childNode, rootNode, nodeMaps), _a));
        }, {})
    };
};
var ItemMap = /** @class */ (function () {
    function ItemMap(props) {
        /**
         * The internal map that we use to keep track
         * of all of the items
         */
        this.map = {};
        this.checkIfOptional = props.checkIfOptional;
    }
    /**
     * Add an item to the cache, along with the path of the node
     * it occurs on
     */
    ItemMap.prototype.addItem = function (itemName, nodePath) {
        if (!this.map[itemName]) {
            this.map[itemName] = {
                events: new Set(),
                states: new Set()
            };
        }
        this.map[itemName].states.add(utils_1.pathToStateValue(nodePath));
    };
    /**
     * Add a triggering event to an item in the cache, for
     * instance the event type which triggers a guard/action/service
     */
    ItemMap.prototype.addEventToItem = function (itemName, eventType, nodePath) {
        this.addItem(itemName, nodePath);
        this.map[itemName].events.add(eventType);
    };
    /**
     * Transform the data into the shape required for index.d.ts
     */
    ItemMap.prototype.toDataShape = function () {
        var _this = this;
        var isRequiredInTotal = false;
        var lines = Object.entries(this.map)
            .filter(function (_a) {
            var name = _a[0];
            return !/\./.test(name);
        })
            .map(function (_a) {
            var name = _a[0], data = _a[1];
            var optional = _this.checkIfOptional(name);
            if (!optional) {
                isRequiredInTotal = true;
            }
            return {
                name: name,
                required: !optional,
                events: Array.from(data.events).filter(Boolean),
                states: Array.from(data.states)
                    .map(function (state) { return JSON.stringify(state); })
                    .filter(Boolean)
            };
        });
        return {
            lines: lines,
            required: isRequiredInTotal
        };
    };
    return ItemMap;
}());
var xstateRegex = /^xstate\./;
exports.introspectMachine = function (machine) {
    var guards = new ItemMap({
        checkIfOptional: function (name) { return Boolean(machine.options.guards[name]); }
    });
    var actions = new ItemMap({
        checkIfOptional: function (name) { return Boolean(machine.options.actions[name]); }
    });
    var services = new ItemMap({
        checkIfOptional: function (name) { return Boolean(machine.options.services[name]); }
    });
    var activities = new ItemMap({
        checkIfOptional: function (name) { return Boolean(machine.options.activities[name]); }
    });
    var nodeMaps = {};
    var allStateNodes = machine.stateIds.map(function (id) {
        return machine.getStateNodeById(id);
    });
    allStateNodes === null || allStateNodes === void 0 ? void 0 : allStateNodes.forEach(function (node) {
        nodeMaps[node.id] = {
            sources: new Set(),
            children: new Set()
        };
    });
    allStateNodes === null || allStateNodes === void 0 ? void 0 : allStateNodes.forEach(function (node) {
        var _a, _b, _c, _d;
        (_a = Object.values(node.states)) === null || _a === void 0 ? void 0 : _a.forEach(function (childNode) {
            nodeMaps[node.id].children.add(childNode.id);
        });
        // TODO - make activities pick up the events
        // that led to them
        (_b = node.activities) === null || _b === void 0 ? void 0 : _b.forEach(function (activity) {
            if (/\./.test(activity.type))
                return;
            if (activity.type && activity.type !== 'xstate.invoke') {
                activities.addItem(activity.type, node.path);
            }
        });
        (_c = node.invoke) === null || _c === void 0 ? void 0 : _c.forEach(function (service) {
            if (typeof service.src !== 'string' || /\./.test(service.src))
                return;
            services.addItem(service.src, node.path);
        });
        (_d = node.transitions) === null || _d === void 0 ? void 0 : _d.forEach(function (transition) {
            var _a, _b, _c;
            (_a = transition.target) === null || _a === void 0 ? void 0 : _a.forEach(function (targetNode) {
                nodeMaps[targetNode.id].sources.add(transition.eventType);
            });
            if (transition.cond && transition.cond.name) {
                if (transition.cond.name !== 'cond') {
                    guards.addEventToItem(transition.cond.name, transition.eventType, node.path);
                }
            }
            (_b = transition.target) === null || _b === void 0 ? void 0 : _b.forEach(function (targetNode) {
                var _a;
                /** Pick up invokes */
                (_a = targetNode.invoke) === null || _a === void 0 ? void 0 : _a.forEach(function (service) {
                    if (typeof service.src !== 'string' || /\./.test(service.src))
                        return;
                    services.addEventToItem(service.src, transition.eventType, node.path);
                });
            });
            if (transition.actions) {
                (_c = transition.actions) === null || _c === void 0 ? void 0 : _c.forEach(function (action) {
                    if (!xstateRegex.test(action.type)) {
                        actions.addEventToItem(action.type, transition.eventType, node.path);
                    }
                    return {
                        name: action.type,
                        event: transition.eventType
                    };
                });
            }
        });
    });
    allStateNodes === null || allStateNodes === void 0 ? void 0 : allStateNodes.forEach(function (node) {
        var _a;
        var allActions = [];
        allActions.push.apply(allActions, node.onExit);
        allActions.push.apply(allActions, node.onEntry);
        allActions === null || allActions === void 0 ? void 0 : allActions.forEach(function (action) {
            if (xstateRegex.test(action.type) || action.exec)
                return;
            actions.addItem(action.type, node.path);
        });
        (_a = node.onEntry) === null || _a === void 0 ? void 0 : _a.forEach(function (action) {
            var sources = nodeMaps[node.id].sources;
            sources === null || sources === void 0 ? void 0 : sources.forEach(function (source) {
                actions.addEventToItem(action.type, source, node.path);
            });
        });
    });
    var subState = makeSubStateFromNode(machine, machine, nodeMaps);
    return {
        stateMatches: exports.getMatchesStates(machine),
        subState: subState,
        guards: guards.toDataShape(),
        actions: actions.toDataShape(),
        services: services.toDataShape(),
        activities: activities.toDataShape()
    };
};
