"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
exports.__esModule = true;
exports.printJsFiles = exports.printToFile = exports.getNodeModulesDir = exports.getDeclarationFileTexts = void 0;
require("colors");
var fs_1 = __importDefault(require("fs"));
var handlebars_1 = __importDefault(require("handlebars"));
var handlebars_helpers_1 = __importDefault(require("handlebars-helpers"));
var path_1 = __importDefault(require("path"));
var pkg_up_1 = __importDefault(require("pkg-up"));
var rimraf_1 = __importDefault(require("rimraf"));
var ensureFolderExists = function (absoluteDir) {
    if (fs_1["default"].existsSync(absoluteDir)) {
        return;
    }
    fs_1["default"].mkdirSync(absoluteDir);
};
var ensureMultipleFoldersExist = function (absoluteRoot, paths) {
    var concatenatedPath = absoluteRoot;
    paths.forEach(function (dir) {
        concatenatedPath = path_1["default"].join(concatenatedPath, dir);
        ensureFolderExists(concatenatedPath);
    });
};
var renderSubstate = function (subState) {
    return "{\n    targets: " + subState.targets + ";\n    sources: " + subState.sources + ";\n    states: {\n      " + Object.entries(subState.states)
        .map(function (_a) {
        var key = _a[0], state = _a[1];
        return key + ": " + renderSubstate(state);
    })
        .join('\n') + "\n    };\n  }";
};
exports.getDeclarationFileTexts = function (cache) {
    var indexTemplateString = fs_1["default"]
        .readFileSync(path_1["default"].resolve(__dirname, './templates/index.d.ts.hbs'))
        .toString();
    var reactTemplateString = fs_1["default"]
        .readFileSync(path_1["default"].resolve(__dirname, './templates/react.d.ts.hbs'))
        .toString();
    handlebars_helpers_1["default"]({
        handlebars: handlebars_1["default"]
    });
    var indexTemplate = handlebars_1["default"].compile(indexTemplateString);
    var reactTemplate = handlebars_1["default"].compile(reactTemplateString);
    var machines = Object.values(cache).map(function (machine) { return ({
        id: machine.id,
        machine: __assign(__assign({}, machine), { subState: renderSubstate(machine.subState) })
    }); });
    return {
        'index.d.ts': indexTemplate({ machines: machines }),
        'react.d.ts': reactTemplate({ machines: machines })
    };
};
exports.getNodeModulesDir = function () {
    var packageJson = pkg_up_1["default"].sync();
    if (!packageJson) {
        throw new Error('Could not find a package.json in any directory in or above this one.');
    }
    var targetDir = path_1["default"].resolve(path_1["default"].dirname(packageJson), 'node_modules');
    return targetDir;
};
exports.printToFile = function (cache, outDir) {
    var files = exports.getDeclarationFileTexts(cache);
    var nodeModulesDir = exports.getNodeModulesDir();
    var targetDir = path_1["default"].resolve(nodeModulesDir, '@xstate/compiled');
    /** Delete @xstate/compiled directory so that it triggers VSCode to re-check it */
    rimraf_1["default"].sync(path_1["default"].join(targetDir, '*.d.ts'));
    exports.printJsFiles();
    ensureMultipleFoldersExist(nodeModulesDir, ['@xstate', 'compiled']);
    fs_1["default"].writeFileSync(outDir
        ? path_1["default"].resolve(process.cwd(), outDir, 'index.d.ts')
        : path_1["default"].join(targetDir, 'index.d.ts'), files['index.d.ts']);
    fs_1["default"].writeFileSync(outDir
        ? path_1["default"].resolve(process.cwd(), outDir, 'react.d.ts')
        : path_1["default"].join(targetDir, 'react.d.ts'), files['react.d.ts']);
    if (outDir) {
        // If the user specifies an outDir, we need to add some dummy types
        // so that we can override something
        fs_1["default"].writeFileSync(path_1["default"].join(targetDir, 'react.d.ts'), "export default any;");
        fs_1["default"].writeFileSync(path_1["default"].join(targetDir, 'index.d.ts'), "export default any;");
    }
};
/**
 * Prints the js files, which needs to be done in advance
 * of rollup looking at the code to ensure there is a module for rollup
 * to statically analyse
 */
exports.printJsFiles = function () {
    var nodeModulesDir = exports.getNodeModulesDir();
    var targetDir = path_1["default"].resolve(nodeModulesDir, '@xstate/compiled');
    ensureMultipleFoldersExist(nodeModulesDir, ['@xstate', 'compiled']);
    var indexJsTemplate = fs_1["default"]
        .readFileSync(path_1["default"].resolve(__dirname, './templates/index.js.hbs'))
        .toString();
    var reactJsTemplate = fs_1["default"]
        .readFileSync(path_1["default"].resolve(__dirname, './templates/react.js.hbs'))
        .toString();
    var packageJsonTemplate = fs_1["default"]
        .readFileSync(path_1["default"].resolve(__dirname, './templates/package.json.hbs'))
        .toString();
    fs_1["default"].writeFileSync(path_1["default"].join(targetDir, 'index.js'), indexJsTemplate);
    fs_1["default"].writeFileSync(path_1["default"].join(targetDir, 'react.js'), reactJsTemplate);
    fs_1["default"].writeFileSync(path_1["default"].join(targetDir, 'package.json'), packageJsonTemplate);
};
