"use strict";
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
exports.__esModule = true;
exports.getRootNode = exports.getMatchesStates = exports.getTransitionsFromNode = void 0;
var utils_1 = require("xstate/lib/utils");
exports.getTransitionsFromNode = function (node) {
    var transitions = new Set();
    if (node.parent) {
        Object.keys(node.parent.states).forEach(function (key) { return transitions.add(key); });
        Object.values(node.parent.states).forEach(function (siblingNode) {
            exports.getMatchesStates(siblingNode).forEach(function (key) {
                var _a;
                if (key === siblingNode.path.join('.')) {
                    return;
                }
                var relativeKey = key;
                if ((((_a = node.parent) === null || _a === void 0 ? void 0 : _a.path.length) || 0) > 0) {
                    relativeKey = relativeKey.replace(new RegExp("^" + node.parent.path.join('.') + "."), '');
                }
                transitions.add(relativeKey);
            });
        });
    }
    Object.values(node.states).map(function (childNode) {
        exports.getMatchesStates(childNode).map(function (key) {
            var _a;
            var relativeKey = key;
            if ((((_a = childNode.parent) === null || _a === void 0 ? void 0 : _a.path.length) || 0) > 0) {
                relativeKey = relativeKey.replace(new RegExp("^" + childNode.parent.path.join('.') + "."), '');
            }
            transitions.add("." + relativeKey);
            transitions.add("" + relativeKey);
        });
    });
    var rootNode = exports.getRootNode(node);
    var nodesWithId = rootNode.stateIds
        .filter(function (id) { return !/(\.|\(machine\))/.test(id); })
        .map(function (id) { return rootNode.getStateNodeById(id); });
    nodesWithId.forEach(function (idNode) {
        exports.getMatchesStates(idNode).forEach(function (match) {
            if (idNode.id === rootNode.id) {
                transitions.add("#" + idNode.id + "." + match);
                return;
            }
            transitions.add(match.replace(new RegExp("^" + idNode.path.join('.')), "#" + idNode.id));
        });
    });
    utils_1.toStatePaths(utils_1.pathToStateValue(node.path)).forEach(function (path) {
        if (path.length > 1) {
            transitions["delete"](path.join('.'));
        }
    });
    return Array.from(transitions);
};
exports.getMatchesStates = function (machine) {
    var allStateNodes = machine.stateIds.map(function (id) {
        return machine.getStateNodeById(id);
    });
    var states = allStateNodes.reduce(function (arr, node) {
        return __spreadArrays(arr, utils_1.toStatePaths(utils_1.pathToStateValue(node.path)).map(function (path) {
            return path.join('.');
        }));
    }, []);
    return states;
};
exports.getRootNode = function (node) {
    if (!node.parent) {
        return node;
    }
    return exports.getRootNode(node.parent);
};
